@@ -39,14 +39,13 @@ import CADETMatch.de as de
 import CADETMatch.de_snooker as de_snooker
 import CADETMatch.stretch as stretch
 
-import json
+import jstyleson
 import shutil
 
 log2 = numpy.log(2)
 
-acceptance_target = 0.234
-acceptance_delta = 0.15
-
+min_acceptance = 0.05
+acceptance_delta = 0.02
 
 def log_previous(cadetValues, kde_previous, kde_previous_scaler):
     # find the right values to use
@@ -74,6 +73,7 @@ def log_likelihood(individual, json_path):
     scores, csv_record, meta_score, results, individual = evo.fitness(individual, json_path)
 
     if results is None:
+        multiprocessing.get_logger().info("log_likelihood results is None %s", individual)
         return -numpy.inf, scores, csv_record, meta_score, results, individual
 
     if results is not None and kde_previous is not None:
@@ -100,6 +100,12 @@ def log_posterior_vectorize(population, json_path, cache, halloffame, meta_hof,
     return results
 
 
+def outside_bounds(x, cache):
+    for i, lb, ub in zip(x, cache.MIN_VALUE, cache.MAX_VALUE):
+        if i < lb or i > ub:
+            return True
+    return False
+
 def log_posterior(x):
     theta, json_path = x
     if json_path != cache.cache.json_path:
@@ -107,9 +113,14 @@ def log_posterior(x):
         util.setupLog(cache.cache.settings["resultsDirLog"], "main.log")
         cache.cache.setup(json_path)
 
+    if outside_bounds(theta, cache.cache):
+        multiprocessing.get_logger().info("individual is outside of range %s", theta)
+        return -numpy.inf, theta, cache.cache.WORST, [], cache.cache.WORST_META, None, theta
+
     ll, scores, csv_record, meta_score, results, individual = log_likelihood(theta, json_path)
     if results is None:
-        return -numpy.inf, None, None, None, None, individual
+        multiprocessing.get_logger().info("log_posterior results is None %s", theta)
+        return -numpy.inf, theta, cache.cache.WORST, [], cache.cache.WORST_META, None, individual
     else:
         return ll, theta, scores, csv_record, meta_score, results, individual
 
@@ -124,9 +135,10 @@ def addChain(*args):
 
 def converged_bounds(chain, length, error_level):
     if chain.shape[1] < (2 * length):
-        return False, None, None
+        return False, None, None, None
     lb = []
     ub = []
+    mid = []
 
     start = chain.shape[1] - length
     stop = chain.shape[1]
@@ -134,16 +146,18 @@ def converged_bounds(chain, length, error_level):
         temp_chain = chain[:, :i, :]
         temp_chain_shape = temp_chain.shape
         temp_chain_flat = temp_chain.reshape(temp_chain_shape[0] * temp_chain_shape[1], temp_chain_shape[2])
-        lb_5, ub_95 = numpy.percentile(temp_chain_flat, [5, 95], 0)
+        lb_5, mid_50, ub_95 = numpy.percentile(temp_chain_flat, [5, 50, 95], 0)
 
         lb.append(lb_5)
         ub.append(ub_95)
+        mid.append(mid_50)
 
     lb = numpy.array(lb)
     ub = numpy.array(ub)
+    mid = numpy.array(mid)
 
     if numpy.all(numpy.std(lb, axis=0) < error_level) and numpy.all(numpy.std(ub, axis=0) < error_level):
-        return True, numpy.mean(lb, axis=0), numpy.mean(ub, axis=0)
+        return True, numpy.mean(lb, axis=0), numpy.mean(mid, axis=0), numpy.mean(ub, axis=0)
     else:
         multiprocessing.get_logger().info(
             "bounds have not yet converged lb min: %s max: %s std: %s  ub min %s max: %s std: %s",
@@ -154,16 +168,16 @@ def converged_bounds(chain, length, error_level):
             numpy.array2string(numpy.max(ub, axis=0), precision=3, separator=","),
             numpy.array2string(numpy.std(ub, axis=0), precision=4, separator=","),
         )
-        return False, None, None
+        return False, None, None, None
 
 
-def rescale(cache, lb, ub, old_lb, old_ub, mcmc_store):
+def rescale(cache, lb, mid, ub, old_lb, old_ub, mcmc_store):
     "give a new lb and ub that will rescale so that the previous lb and ub takes up about 1/2 of the search width"
     new_size = len(lb)
     old_lb_slice = old_lb[:new_size]
     old_ub_slice = old_ub[:new_size]
 
-    center = (ub + lb) / 2.0
+    center = mid
     old_center = (old_ub + old_lb) / 2.0
 
     new_lb = lb - 2 * (center - lb)
@@ -249,7 +263,7 @@ def change_bounds_json(cache, lb, ub, mcmc_store):
     new_settings_file = settings_file.with_name(new_name)
 
     with settings_file.open() as json_data:
-        settings = json.load(json_data)
+        settings = jstyleson.load(json_data)
 
         idx = 0
         for parameter in settings["parameters"]:
@@ -263,9 +277,9 @@ def change_bounds_json(cache, lb, ub, mcmc_store):
                 idx = idx + count
 
         with new_settings_file.open(mode="w") as json_data:
-            json.dump(settings, json_data, indent=4, sort_keys=False)
+            jstyleson.dump(settings, json_data, indent=4, sort_keys=False)
 
-        mcmc_store.root.bounds_change.json = json.dumps(settings["parameters"], sort_keys=False)
+        mcmc_store.root.bounds_change.json = jstyleson.dumps(settings["parameters"], sort_keys=False)
 
     # copy the original file to a backup name
     shutil.copy(settings_file, settings_file_backup)
@@ -273,7 +287,7 @@ def change_bounds_json(cache, lb, ub, mcmc_store):
     # copy over our new settings file to the original file also
     # this is so that external programs also see the new bounds
     with settings_file.open(mode="w") as json_data:
-        json.dump(settings, json_data, indent=4, sort_keys=False)
+        jstyleson.dump(settings, json_data, indent=4, sort_keys=False)
 
     return new_settings_file.as_posix()
 
@@ -354,14 +368,14 @@ def sampler_auto_bounds(cache, checkpoint, sampler, checkpointFile, mcmc_store):
         util.graph_corner_process(cache, last=False)
 
         if generation % checkInterval == 0:
-            converged, lb, ub = converged_bounds(bounds_chain[:, :, :new_parameters], 200, 1e-3)
+            converged, lb, mid, ub = converged_bounds(bounds_chain[:, :, :new_parameters], 200, 1e-3)
 
             if converged:
                 # sys.exit()
                 finished = True
 
                 new_min_value, center, new_max_value = rescale(
-                    cache, lb, ub, numpy.array(cache.MIN_VALUE), numpy.array(cache.MAX_VALUE), mcmc_store
+                    cache, lb, mid, ub, numpy.array(cache.MIN_VALUE), numpy.array(cache.MAX_VALUE), mcmc_store
                 )
 
                 json_path = change_bounds_json(cache, new_min_value, new_max_value, mcmc_store)
@@ -492,7 +506,7 @@ def sampler_burn(cache, checkpoint, sampler, checkpointFile, mcmc_store):
 
         if numpy.std(converge_real) < tol and len(converge) == len(converge_real):
             average_converge = numpy.mean(converge)
-            if (acceptance_target - acceptance_delta) < average_converge < (acceptance_target + acceptance_delta):
+            if average_converge > min_acceptance:
                 multiprocessing.get_logger().info("burn in completed at iteration %s", generation)
                 finished = True
 
@@ -501,7 +515,7 @@ def sampler_burn(cache, checkpoint, sampler, checkpointFile, mcmc_store):
                 finished = True
 
             if not finished:
-                new_distance = numpy.abs(average_converge - acceptance_target)
+                new_distance = min_acceptance - average_converge
                 if new_distance < distance:
                     distance_n = sampler._moves[1].n
                     distance = new_distance
@@ -511,23 +525,13 @@ def sampler_burn(cache, checkpoint, sampler, checkpointFile, mcmc_store):
                     )
                     converge[:] = numpy.nan
                     prev_n = sampler._moves[1].n
-                    if average_converge > (acceptance_target + 3 * acceptance_delta):
-                        # n must be increased to decrease the acceptance rate (step size)
-                        power += 4
-                    elif average_converge > (acceptance_target + 2 * acceptance_delta):
-                        # n must be increased to decrease the acceptance rate (step size)
-                        power += 2
-                    elif average_converge > (acceptance_target + 1 * acceptance_delta):
-                        # n must be increased to decrease the acceptance rate (step size)
-                        power += 1
-                    elif average_converge < (acceptance_target - 3 * acceptance_delta):
+                    if average_converge < (min_acceptance - 2 * acceptance_delta):
                         # n must be decreased to increase the acceptance rate (step size)
-                        power -= 4
-                    elif average_converge < (acceptance_target - 2 * acceptance_delta):
+                        power -= 3
+                    elif average_converge < (min_acceptance - acceptance_delta):
                         # n must be decreased to increase the acceptance rate (step size)
                         power -= 2
-                    elif average_converge < (acceptance_target - 1 * acceptance_delta):
-                        # n must be decreased to increase the acceptance rate (step size)
+                    else:
                         power -= 1
                     new_n = power
                     sampler._moves[1].n = power
@@ -859,7 +863,7 @@ def mle_process(last=False, interval=3600):
         mle_process.last_time = time.time()
 
 
-def get_population(base, size, diff=0.05):
+def get_population(base, size, diff=0.1):
     new_population = base
     row, col = base.shape
     multiprocessing.get_logger().info("%s", base)
@@ -1001,21 +1005,26 @@ def process(population_order, cache, halloffame, meta_hof, grad_hof, progress_ho
 
     log_likelihoods_lookup = {}
 
+    keep = set()
+
     for ll, theta, fit, csv_line, meta_score, result, individual in results:
         log_likelihoods_lookup[tuple(individual)] = float(ll)
-        if result is not None:
-            parameters = theta
+        if len(csv_line):
+            #if csv_line is blank it means a simulation failed or is out of bounds, if this is handed on for
+            #processing it will cause problems, failed simulations are arleady recorded and we don't want their
+            #failure data in the recorers where other stuff picks it up
+            keep.add(tuple(individual))            
+
             fitnesses_lookup[tuple(individual)] = (fit, csv_line, meta_score, result, tuple(individual))
 
-            ind = cache.toolbox.individual_guess(parameters)
+            ind = cache.toolbox.individual_guess(individual)
             population_lookup[tuple(individual)] = ind
 
-            # result_data['mcmc_score'].append(ll)
-
     # everything above is async (unordered) and needs to be reordered based on the population_order
-    population = [population_lookup[tuple(row)] for row in population_order]
-    fitnesses = [fitnesses_lookup[tuple(row)] for row in population_order]
-    log_likelihoods = [log_likelihoods_lookup[tuple(row)] for row in population_order]
+    population = [population_lookup[tuple(row)] for row in population_order if tuple(row) in keep]
+    fitnesses = [fitnesses_lookup[tuple(row)] for row in population_order if tuple(row) in keep]
+    log_likelihoods = [log_likelihoods_lookup[tuple(row)] for row in population_order if tuple(row) in keep]
+    log_likelihoods_all = [log_likelihoods_lookup[tuple(row)] for row in population_order]
 
     stalled, stallWarn, progressWarn = util.process_population(
         cache.toolbox, cache, population, fitnesses, writer, csv_file, halloffame, meta_hof, progress_hof, process.gen, result_data
@@ -1033,14 +1042,14 @@ def process(population_order, cache, halloffame, meta_hof, grad_hof, progress_ho
         process.generation_start,
         result_data,
         line_log=False,
-        probability=numpy.exp(log_likelihoods),
+        probability=numpy.exp(log_likelihoods)
     )
 
     util.graph_process(cache, process.gen)
 
     process.gen += 1
     process.generation_start = time.time()
-    return log_likelihoods
+    return log_likelihoods_all
 
 
 def process_chain(chain, cache, idx):
